package com.example.nto.service.impl;

import com.example.nto.model.entity.Employee;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.exception.CodeNotFoundException;
import com.example.nto.service.exception.EmployeeNotFoundException;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService, UserDetailsService {

    private final EmployeeRepository employeeRepository;

    private final PasswordEncoder passwordEncoder;

    private final CodeRepository codeRepository;

    private final AuthenticationManager authenticationManager;

    @Override
    public void auth(final String login, final String password) {
        final Employee employee = getEmployee(login);
        authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(employee.getUsername(), employee.getPassword()));
    }

    @Override
    public Employee getEmployee(final String login) {
        if (!employeeRepository.existsByLogin(login)) {
            throw new EmployeeNotFoundException();
        }
        return employeeRepository.findEmployeeByLogin(login);
    }

    @Override
    public void updateVisit(final String login, final long value) {
        if (!employeeRepository.existsByLogin(login)) {
            throw new EmployeeNotFoundException();
        }
        if (!codeRepository.existsByValue(value)) {
            throw new CodeNotFoundException();
        }
        final Employee employee = employeeRepository.findEmployeeByLogin(login);
        employee.setLastVisit(LocalDateTime.now());
        employeeRepository.save(employee);
    }

    @Override
    public void addEmployee(Employee employee) {
        employee.setPassword(passwordEncoder.encode(employee.getPassword()));
        employeeRepository.save(employee);
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        return employeeRepository.findEmployeeByLogin(username);
    }
}
